@@ -9,7 +9,7 @@ use strict;
 use warnings;
 no warnings 'recursion'; # Disable the "deep recursion" warning
 
-our $VERSION = '0.36';
+our $VERSION = '0.37';
 
 use Carp qw(); # don't import croak
 use Scalar::Util qw( weaken blessed reftype );
@@ -1523,6 +1523,7 @@ sub without_cancel
    });
 
    $new->{orig} = $self; # just to strongref it - RT122920
+   $new->on_ready( sub { undef $_[0]->{orig} } );
 
    return $new;
 }
@@ -1640,13 +1641,13 @@ sub wait_all
 
    weaken( my $weakself = $self );
    my $sub_on_ready = sub {
-      return unless $weakself;
+      return unless my $self = $weakself;
 
       $pending--;
       $pending and return;
 
-      $weakself->{result} = [ @subs ];
-      $weakself->_mark_ready( "wait_all" );
+      $self->{result} = [ @subs ];
+      $self->_mark_ready( "wait_all" );
    };
 
    foreach my $sub ( @subs ) {
@@ -1713,26 +1714,26 @@ sub wait_any
 
    weaken( my $weakself = $self );
    my $sub_on_ready = sub {
-      return unless $weakself;
-      return if $weakself->{result} or $weakself->{failure}; # don't recurse on child ->cancel
+      return unless my $self = $weakself;
+      return if $self->{result} or $self->{failure}; # don't recurse on child ->cancel
 
       return if --$pending and $_[0]->{cancelled};
 
       if( $_[0]->{cancelled} ) {
-         $weakself->{failure} = [ "All component futures were cancelled" ];
+         $self->{failure} = [ "All component futures were cancelled" ];
       }
       elsif( $_[0]->{failure} ) {
-         $weakself->{failure} = [ $_[0]->failure ];
+         $self->{failure} = [ $_[0]->failure ];
       }
       else {
-         $weakself->{result}  = [ $_[0]->get ];
+         $self->{result}  = [ $_[0]->get ];
       }
 
       foreach my $sub ( @subs ) {
          $sub->{ready} or $sub->cancel;
       }
 
-      $weakself->_mark_ready( "wait_any" );
+      $self->_mark_ready( "wait_any" );
    };
 
    foreach my $sub ( @subs ) {
@@ -1808,29 +1809,29 @@ sub needs_all
 
    weaken( my $weakself = $self );
    my $sub_on_ready = sub {
-      return unless $weakself;
-      return if $weakself->{result} or $weakself->{failure}; # don't recurse on child ->cancel
+      return unless my $self = $weakself;
+      return if $self->{result} or $self->{failure}; # don't recurse on child ->cancel
 
       if( $_[0]->{cancelled} ) {
-         $weakself->{failure} = [ "A component future was cancelled" ];
+         $self->{failure} = [ "A component future was cancelled" ];
          foreach my $sub ( @subs ) {
             $sub->cancel if !$sub->{ready};
          }
-         $weakself->_mark_ready( "needs_all" );
+         $self->_mark_ready( "needs_all" );
       }
       elsif( my @failure = $_[0]->failure ) {
-         $weakself->{failure} = \@failure;
+         $self->{failure} = \@failure;
          foreach my $sub ( @subs ) {
             $sub->cancel if !$sub->{ready};
          }
-         $weakself->_mark_ready( "needs_all" );
+         $self->_mark_ready( "needs_all" );
       }
       else {
          $pending--;
          $pending and return;
 
-         $weakself->{result} = [ map { $_->get } @subs ];
-         $weakself->_mark_ready( "needs_all" );
+         $self->{result} = [ map { $_->get } @subs ];
+         $self->_mark_ready( "needs_all" );
       }
    };
 
@@ -1916,27 +1917,27 @@ sub needs_any
 
    weaken( my $weakself = $self );
    my $sub_on_ready = sub {
-      return unless $weakself;
-      return if $weakself->{result} or $weakself->{failure}; # don't recurse on child ->cancel
+      return unless my $self = $weakself;
+      return if $self->{result} or $self->{failure}; # don't recurse on child ->cancel
 
       return if --$pending and $_[0]->{cancelled};
 
       if( $_[0]->{cancelled} ) {
-         $weakself->{failure} = [ "All component futures were cancelled" ];
-         $weakself->_mark_ready( "needs_any" );
+         $self->{failure} = [ "All component futures were cancelled" ];
+         $self->_mark_ready( "needs_any" );
       }
       elsif( my @failure = $_[0]->failure ) {
          $pending and return;
 
-         $weakself->{failure} = \@failure;
-         $weakself->_mark_ready( "needs_any" );
+         $self->{failure} = \@failure;
+         $self->_mark_ready( "needs_any" );
       }
       else {
-         $weakself->{result} = [ $_[0]->get ];
+         $self->{result} = [ $_[0]->get ];
          foreach my $sub ( @subs ) {
             $sub->cancel if !$sub->{ready};
          }
-         $weakself->_mark_ready( "needs_any" );
+         $self->_mark_ready( "needs_any" );
       }
    };
 
